@@ -401,53 +401,6 @@ error_free:
   return NULL;
 }
 
-static girara_debug_level_t debug_level = GIRARA_DEBUG;
-
-void
-_girara_debug(const char* function, int line, girara_debug_level_t level, const char* format, ...)
-{
-  if (level < debug_level) {
-    return;
-  }
-
-  switch (level)
-  {
-    case GIRARA_WARNING:
-      fprintf(stderr, "warning: ");
-      break;
-    case GIRARA_ERROR:
-      fprintf(stderr, "error: ");
-      break;
-    case GIRARA_INFO:
-      fprintf(stderr, "info: ");
-      break;
-    case GIRARA_DEBUG:
-      fprintf(stderr, "debug: (%s:%d) ", function, line);
-      break;
-    default:
-      return;
-  }
-
-  va_list ap;
-  va_start(ap, format);
-  vfprintf(stderr, format, ap);
-  va_end(ap);
-
-  fprintf(stderr, "\n");
-}
-
-girara_debug_level_t
-girara_get_debug_level()
-{
-  return debug_level;
-}
-
-void
-girara_set_debug_level(girara_debug_level_t level)
-{
-  debug_level = level;
-}
-
 void
 update_state_by_keyval(int *state, int keyval)
 {
@@ -483,6 +436,51 @@ girara_escape_string(const char* value)
   return g_string_free(str, FALSE);
 }
 
+char*
+girara_replace_substring(const char* string, const char* old, const char* new)
+{
+  if (string == NULL || old == NULL || new == NULL) {
+    return NULL;
+  }
+
+  size_t old_len = strlen(old);
+  size_t new_len = strlen(new);
+
+  /* count occurrences */
+  size_t count = 0;
+  size_t i = 0;
+
+  for (i = 0; string[i] != '\0'; i++) {
+    if (strstr(&string[i], old) == &string[i]) {
+      i += (old_len - 1);
+      count++;
+    }
+  }
+
+  if (count == 0) {
+    return g_strdup(string);
+  }
+
+  char* ret = g_try_malloc0(sizeof(char) * (i - count * old_len + count * new_len + 1));
+  if (ret == NULL) {
+    return NULL;
+  }
+
+  /* replace */
+  char* iter = ret;
+  while (*string != '\0') {
+    if (strstr(string, old) == string) {
+      strncpy(iter, new, new_len);
+      iter += new_len;
+      string += old_len;
+    } else {
+      *iter++ = *string++;
+    }
+  }
+
+  return ret;
+}
+
 bool
 girara_exec_with_argument_list(girara_session_t* session, girara_list_t* argument_list)
 {
@@ -525,3 +523,15 @@ girara_exec_with_argument_list(girara_session_t* session, girara_list_t* argumen
 
   return ret;
 }
+
+void
+widget_add_class(GtkWidget* widget, const char* styleclass)
+{
+  if (widget == NULL || styleclass == NULL) {
+    return;
+  }
+
+  GtkStyleContext* context = gtk_widget_get_style_context(widget);
+  gtk_style_context_add_class(context, styleclass);
+}
+
